@@ -7,20 +7,18 @@
 
 // cSpell:ignore wc wc's
 
-#[macro_use]
-extern crate uucore;
-
 mod count_fast;
 mod countable;
 mod word_count;
+use clap::builder::ValueParser;
 use count_fast::{count_bytes_chars_and_lines_fast, count_bytes_fast};
 use countable::WordCountable;
 use unicode_width::UnicodeWidthChar;
 use utf8::{BufReadDecoder, BufReadDecoderError};
-use uucore::format_usage;
+use uucore::{format_usage, show};
 use word_count::{TitledWordCount, WordCount};
 
-use clap::{crate_version, Arg, ArgMatches, Command};
+use clap::{crate_version, Arg, ArgAction, ArgMatches, Command};
 
 use std::cmp::max;
 use std::error::Error;
@@ -54,17 +52,17 @@ impl Settings {
             show_control: false,
         };
 
-        let files0_from_stdin_mode = match matches.value_of(options::FILES0_FROM) {
+        let files0_from_stdin_mode = match matches.get_one::<String>(options::FILES0_FROM) {
             Some(files_0_from) => files_0_from == STDIN_REPR,
             None => false,
         };
 
         let settings = Self {
-            show_bytes: matches.contains_id(options::BYTES),
-            show_chars: matches.contains_id(options::CHAR),
-            show_lines: matches.contains_id(options::LINES),
-            show_words: matches.contains_id(options::WORDS),
-            show_max_line_length: matches.contains_id(options::MAX_LINE_LENGTH),
+            show_bytes: matches.get_flag(options::BYTES),
+            show_chars: matches.get_flag(options::CHAR),
+            show_lines: matches.get_flag(options::LINES),
+            show_words: matches.get_flag(options::WORDS),
+            show_max_line_length: matches.get_flag(options::MAX_LINE_LENGTH),
             files0_from_stdin_mode,
             title_quoting_style,
         };
@@ -195,7 +193,7 @@ impl Display for WcError {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let matches = uu_app().get_matches_from(args);
+    let matches = uu_app().try_get_matches_from(args)?;
 
     let inputs = inputs(&matches)?;
 
@@ -204,7 +202,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     wc(&inputs, &settings)
 }
 
-pub fn uu_app<'a>() -> Command<'a> {
+pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
@@ -214,18 +212,19 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(options::BYTES)
                 .short('c')
                 .long(options::BYTES)
-                .help("print the byte counts"),
+                .help("print the byte counts")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::CHAR)
                 .short('m')
                 .long(options::CHAR)
-                .help("print the character counts"),
+                .help("print the character counts")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FILES0_FROM)
                 .long(options::FILES0_FROM)
-                .takes_value(true)
                 .value_name("F")
                 .help(
                     "read input from the files specified by
@@ -238,25 +237,27 @@ pub fn uu_app<'a>() -> Command<'a> {
             Arg::new(options::LINES)
                 .short('l')
                 .long(options::LINES)
-                .help("print the newline counts"),
+                .help("print the newline counts")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::MAX_LINE_LENGTH)
                 .short('L')
                 .long(options::MAX_LINE_LENGTH)
-                .help("print the length of the longest line"),
+                .help("print the length of the longest line")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::WORDS)
                 .short('w')
                 .long(options::WORDS)
-                .help("print the word counts"),
+                .help("print the word counts")
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(ARG_FILES)
-                .multiple_occurrences(true)
-                .takes_value(true)
-                .allow_invalid_utf8(true)
+                .action(ArgAction::Append)
+                .value_parser(ValueParser::os_string())
                 .value_hint(clap::ValueHint::FilePath),
         )
 }
@@ -273,7 +274,7 @@ fn inputs(matches: &ArgMatches) -> UResult<Vec<Input>> {
 
             Ok(os_values.map(|s| Input::from(s.as_os_str())).collect())
         }
-        None => match matches.value_of(options::FILES0_FROM) {
+        None => match matches.get_one::<String>(options::FILES0_FROM) {
             Some(files_0_from) => create_paths_from_files0(files_0_from),
             None => Ok(vec![Input::Stdin(StdinKind::Implicit)]),
         },
@@ -316,6 +317,7 @@ fn word_count_from_reader<T: WordCountable>(
         // Specialize scanning loop to improve the performance.
         (false, false, false, false, false) => unreachable!(),
 
+        // show_bytes
         (true, false, false, false, false) => {
             // Fast path when only show_bytes is true.
             let (bytes, error) = count_bytes_fast(&mut reader);
@@ -329,58 +331,75 @@ fn word_count_from_reader<T: WordCountable>(
         }
 
         // Fast paths that can be computed without Unicode decoding.
+        // show_lines
         (false, false, true, false, false) => {
             count_bytes_chars_and_lines_fast::<_, false, false, true>(&mut reader)
         }
+        // show_chars
         (false, true, false, false, false) => {
             count_bytes_chars_and_lines_fast::<_, false, true, false>(&mut reader)
         }
+        // show_chars, show_lines
         (false, true, true, false, false) => {
             count_bytes_chars_and_lines_fast::<_, false, true, true>(&mut reader)
         }
+        // show_bytes, show_lines
         (true, false, true, false, false) => {
             count_bytes_chars_and_lines_fast::<_, true, false, true>(&mut reader)
         }
+        // show_bytes, show_chars
         (true, true, false, false, false) => {
             count_bytes_chars_and_lines_fast::<_, true, true, false>(&mut reader)
         }
+        // show_bytes, show_chars, show_lines
         (true, true, true, false, false) => {
             count_bytes_chars_and_lines_fast::<_, true, true, true>(&mut reader)
         }
-
+        // show_words
         (_, false, false, false, true) => {
             word_count_from_reader_specialized::<_, false, false, false, true>(reader)
         }
+        // show_max_line_length
         (_, false, false, true, false) => {
             word_count_from_reader_specialized::<_, false, false, true, false>(reader)
         }
+        // show_max_line_length, show_words
         (_, false, false, true, true) => {
             word_count_from_reader_specialized::<_, false, false, true, true>(reader)
         }
+        // show_chars, show_words
         (_, false, true, false, true) => {
             word_count_from_reader_specialized::<_, false, true, false, true>(reader)
         }
+        // show_chars, show_lines
         (_, false, true, true, false) => {
             word_count_from_reader_specialized::<_, false, true, true, false>(reader)
         }
+        // show_lines, show_max_line_length, show_words
         (_, false, true, true, true) => {
             word_count_from_reader_specialized::<_, false, true, true, true>(reader)
         }
+        // show_chars, show_words
         (_, true, false, false, true) => {
             word_count_from_reader_specialized::<_, true, false, false, true>(reader)
         }
+        // show_chars, show_max_line_length
         (_, true, false, true, false) => {
             word_count_from_reader_specialized::<_, true, false, true, false>(reader)
         }
+        // show_chars, show_max_line_length, show_words
         (_, true, false, true, true) => {
             word_count_from_reader_specialized::<_, true, false, true, true>(reader)
         }
+        // show_chars, show_lines, show_words
         (_, true, true, false, true) => {
             word_count_from_reader_specialized::<_, true, true, false, true>(reader)
         }
+        // show_chars, show_lines, show_max_line_length
         (_, true, true, true, false) => {
             word_count_from_reader_specialized::<_, true, true, true, false>(reader)
         }
+        // show_chars, show_lines, show_max_line_length, show_words
         (_, true, true, true, true) => {
             word_count_from_reader_specialized::<_, true, true, true, true>(reader)
         }
